@@ -1,8 +1,8 @@
 /*
 
 
-  Jacksum 3.4.0 - a checksum utility in Java
-  Copyright (c) 2001-2022 Dipl.-Inf. (FH) Johann N. Löfflmann,
+  Jacksum 3.6.0 - a checksum utility in Java
+  Copyright (c) 2001-2023 Dipl.-Inf. (FH) Johann N. Löfflmann,
   All Rights Reserved, <https://jacksum.net>.
 
   This program is free software: you can redistribute it and/or modify it under
@@ -27,8 +27,8 @@ import java.security.NoSuchAlgorithmException;
 
 public class Fnv1a_n extends Fnv1_n {
 
-    public Fnv1a_n(String strBitWidth) throws NoSuchAlgorithmException {
-        super(strBitWidth);
+    public Fnv1a_n(String width) throws NoSuchAlgorithmException {
+        super(width);
     }
 
     @Override
@@ -36,7 +36,7 @@ public class Fnv1a_n extends Fnv1_n {
         for (int i = offset; i < length + offset; i++) {
             value = value.xor(BIG[bytes[i] & 0xFF]);
             value = value.multiply(prime);
-            value = value.mod(modulo);
+            value = value.and(mask); // in Java if x >= 0: x % 2^n == x & (2^n - 1)
         }
         this.length += length;
     }
