@@ -1,8 +1,8 @@
 /*
 
 
-  Jacksum 3.4.0 - a checksum utility in Java
-  Copyright (c) 2001-2022 Dipl.-Inf. (FH) Johann N. Löfflmann,
+  Jacksum 3.7.0 - a checksum utility in Java
+  Copyright (c) 2001-2023 Dipl.-Inf. (FH) Johann N. Löfflmann,
   All Rights Reserved, <https://jacksum.net>.
 
   This program is free software: you can redistribute it and/or modify it under
@@ -22,7 +22,7 @@
  */
 package net.jacksum.cli;
 
-import net.jacksum.actions.help.Help;
+import net.jacksum.actions.info.help.Help;
 import net.jacksum.algorithms.AbstractChecksum;
 import net.jacksum.formats.Encoding;
 import net.jacksum.multicore.ThreadControl;
@@ -30,6 +30,8 @@ import net.jacksum.parameters.ParameterException;
 import net.jacksum.parameters.Parameters;
 import net.loefflmann.sugar.util.GeneralString;
 
+import java.util.Locale;
+
 /**
  * All parameters for the Command Line Interface (CLI)
  *
@@ -66,6 +68,7 @@ public class CLIParameters {
     public static final String __DONT_FOLLOW_SYMLINKS_TO_DIRECTORIES = "--dont-follow-symlinks-to-directories";
     public static final String _EXPECT_HASH = "-e";
     public static final String __EXPECT_HASH = "--expect-hash";
+    public static final String __EXPECT = "--expect";
     public static final String _ENCODING = "-E";
     public static final String __ENCODING = "--encoding";
     public static final String _DONT_FOLLOW_SYMLINKS_TO_FILES = "-f";
@@ -80,13 +83,18 @@ public class CLIParameters {
     public static final String __GROUP_BYTES_SEPARATOR = "--group-bytes-separator";
     public static final String _HELP = "-h";
     public static final String __HELP = "--help";
+    public static final String __HMACS = "--hmacs";
     public static final String __INFO = "--info";
     public static final String _IGNORE_LINES_STARTING_WITH_STRING = "-I";
     public static final String __IGNORE_LINES_STARTING_WITH_STRING = "--ignore-lines-starting-with-string";
+    public static final String __IGNORE_EMPTY_LINES = "--ignore-empty-lines";
+    public static final String _KEY = "-k";
+    public static final String __KEY = "--key";
     public static final String _LIST = "-l";
     public static final String __LIST = "--list";
     public static final String __LIST_FILTER = "--list-filter";
     public static final String __WANTED_LIST_FILTER = "--wanted-list-filter";
+    public static final String __MATCH_FILTER = "--match-filter";
     public static final String __LEGACY_STDIN_NAME = "--legacy-stdin-name";
     public static final String _FILE_LIST = "-L";
     public static final String __FILE_LIST = "--file-list";
@@ -96,6 +104,7 @@ public class CLIParameters {
     public static final String __OUTPUT_FILE = "--output-file";
     public static final String _OUTPUT_FILE_OVERWRITE = "-O";
     public static final String __OUTPUT_FILE_OVERWRITE = "--output-file-overwrite";
+    public static final String __OUTPUT_FILE_REPLACE_TOKENS = "--output-file-replace-tokens";
     public static final String _PATH_SEPARATOR = "-P";
     public static final String __PATH_SEPARATOR = "--path-separator";
     public static final String _QUICK = "-q";
@@ -106,6 +115,7 @@ public class CLIParameters {
     public static final String __SCAN_NTFS_ADS = "--scan-ntfs-ads";
     public static final String _SEPARATOR = "-s";
     public static final String __SEPARATOR = "--separator";
+    public static final String __STRING_LIST = "--string-list";
     public static final String __THREADS_HASHING = "--threads-hashing";
     public static final String __THREADS_READING =  "--threads-reading";
     public static final String _TIMESTAMP = "-t";
@@ -135,6 +145,11 @@ public class CLIParameters {
     public static final String __CHARSET_WANTED_LIST = "--charset-wanted-list";
     public static final String __WANTED_LIST_CHARSET = "--wanted-list-charset";
 
+    public static final String __CHARSET_STRING_LIST = "--charset-string-list";
+    public static final String __STRING_LIST_CHARSET = "--string-list-charset";
+    public static final String __CHARSET_CONSOLE = "--charset-console";
+    public static final String __CONSOLE_CHARSET = "--console-charset";
+
     public static final String __CHARSET_STDOUT = "--charset-stdout";
     public static final String __STDOUT_CHARSET = "--stdout-charset";
     public static final String __CHARSET_STDERR = "--charset-stderr";
@@ -277,7 +292,7 @@ public class CLIParameters {
                 } else if (arg.equals(_DONT_FOLLOW_SYMLINKS_TO_DIRECTORIES) || arg.equals(__DONT_FOLLOW_SYMLINKS_TO_DIRECTORIES)) {
                     parameters.setDontFollowSymlinksToDirectories(true);
 
-                } else if (arg.equals(_EXPECT_HASH) || (arg.equals(__EXPECT_HASH))) {
+                } else if (arg.equals(_EXPECT_HASH) || (arg.equals(__EXPECT_HASH)) || arg.equals(__EXPECT)) {
                     if (firstfile < args.length) {
                         parameters.setExpected(args[firstfile++]);
                     } else {
@@ -376,6 +391,9 @@ public class CLIParameters {
                         }
                     }
 
+                } else if (arg.equals(__HMACS)) {
+                    parameters.setHMACsWanted(true);
+
                 } else if (arg.equals(__INFO)) {
                     parameters.setInfoMode(true);
 
@@ -386,6 +404,21 @@ public class CLIParameters {
                         handleUserParamError(arg, __IGNORE_LINES_STARTING_WITH_STRING);
                     }
 
+                } else if (arg.equals(__IGNORE_EMPTY_LINES)) {
+                    parameters.setIgnoreEmptyLines(true);
+
+                } else if (arg.equals(_KEY) || arg.equals(__KEY)) {
+                    if (firstfile < args.length) {
+                        arg = args[firstfile++];
+                        try {
+                            parameters.setKey(arg);
+                        } catch (IllegalArgumentException e) {
+                            throw new ParameterException(e.getMessage());
+                        }
+                    } else {
+                        handleUserParamError(arg, __KEY);
+                    }
+
                 } else if (arg.equals(_LIST) || arg.equals(__LIST)) {
                     parameters.setList(true);
 
@@ -401,19 +434,18 @@ public class CLIParameters {
                         handleUserParamError(arg, __LIST_FILTER);
                     }
 
-                } else if (arg.equals(__WANTED_LIST_FILTER)) {
+                } else if (arg.equals(__MATCH_FILTER) || arg.equals(__WANTED_LIST_FILTER)) {
                     if (firstfile < args.length) {
                         try {
                             parameters.getWantedListFilter().setFilter(args[firstfile++]);
                         } catch (IllegalArgumentException e) {
-                            Help.printHelp(HELP_DEFAULT_LANGUAGE, __WANTED_LIST_FILTER);
+                            Help.printHelp(HELP_DEFAULT_LANGUAGE, __MATCH_FILTER);
                             throw new ParameterException(e.getMessage());
                         }
                     } else {
-                        handleUserParamError(arg, __WANTED_LIST_FILTER);
+                        handleUserParamError(arg, __MATCH_FILTER);
                     }
 
-
                 } else if (arg.equals(__LEGACY_STDIN_NAME)) {
                     parameters.setStdinName("-");
                     AbstractChecksum.setStdinName("-");
@@ -455,6 +487,9 @@ public class CLIParameters {
                         handleUserParamError(arg, __OUTPUT_FILE_OVERWRITE);
                     }
 
+                } else if (arg.equals(__OUTPUT_FILE_REPLACE_TOKENS)) {
+                    parameters.setOutputFileReplaceTokens(true);
+
                 } else if (arg.equals(_PATH_SEPARATOR) || arg.equals(__PATH_SEPARATOR)) {
                     if (firstfile < args.length) {
                         arg = args[firstfile++];
@@ -513,6 +548,13 @@ public class CLIParameters {
                         handleUserParamError(arg, __SEPARATOR);
                     }
 
+                } else if (arg.equals(__STRING_LIST)) {
+                    if (firstfile < args.length) {
+                        parameters.setStringList(args[firstfile++]);
+                    } else {
+                        handleUserParamError(arg, __STRING_LIST);
+                    }
+
                 } else if (arg.equals(__THREADS_HASHING)) {
                     if (firstfile < args.length) {
                         arg = args[firstfile++];
@@ -643,6 +685,20 @@ public class CLIParameters {
                         handleUserParamError(arg, __CHARSET_OUTPUT_FILE);
                     }
 
+                } else if (arg.equals(__CHARSET_STRING_LIST) || arg.equals(__STRING_LIST_CHARSET)) {
+                    if (firstfile < args.length) {
+                        parameters.setCharsetStringList(args[firstfile++]);
+                    } else {
+                        handleUserParamError(arg, __CHARSET_STRING_LIST);
+                    }
+
+                } else if (arg.equals(__CHARSET_CONSOLE) || arg.equals(__CONSOLE_CHARSET)) {
+                    if (firstfile < args.length) {
+                        parameters.setCharsetConsole(args[firstfile++]);
+                    } else {
+                        handleUserParamError(arg, __CHARSET_CONSOLE);
+                    }
+
                 } else if (arg.equals(__CHARSET_STDOUT) || arg.equals(__STDOUT_CHARSET)) {
                     if (firstfile < args.length) {
                         parameters.setCharsetStdout(args[firstfile++]);
@@ -666,6 +722,7 @@ public class CLIParameters {
             // if --info has been set, enable a potentially disabled verbose object again
             if (parameters.isInfoMode()) {
                 parameters.getVerbose().setDefault();
+                parameters.getVerbose().setInfo(true);
             }
 
             // parsing the verboseControl at the end of the while loop,
@@ -688,6 +745,22 @@ public class CLIParameters {
                 }
             }
 
+            // replace #ALGONAME, #ALGONAME{lowercase}, #ALGONAME{uppercase}
+            if (parameters.getOutputFile() != null && parameters.isOutputFileReplaceTokens()) {
+                parameters.setOutputFileRaw(parameters.getOutputFile());
+                if (parameters.getAlgorithm() != null) {
+                    parameters.setOutputFile(
+                            parameters.getOutputFile().replaceAll(
+                                    "#ALGONAME\\{uppercase\\}", parameters.getAlgorithm().toUpperCase(Locale.US).replace(':', '=')));
+                    parameters.setOutputFile(
+                            parameters.getOutputFile().replaceAll(
+                                    "#ALGONAME\\{lowercase\\}", "#ALGONAME"));
+                    parameters.setOutputFile(
+                            parameters.getOutputFile().replaceAll(
+                                    "#ALGONAME", parameters.getAlgorithm().toLowerCase(Locale.US).replace(':', '=')));
+                }
+            }
+
         } // end-if args.length > 0
         // end parsing arguments
         return parameters;
