@@ -1,8 +1,8 @@
 /*
 
 
-  Jacksum 3.4.0 - a checksum utility in Java
-  Copyright (c) 2001-2022 Dipl.-Inf. (FH) Johann N. Löfflmann,
+  Jacksum 3.6.0 - a checksum utility in Java
+  Copyright (c) 2001-2023 Dipl.-Inf. (FH) Johann N. Löfflmann,
   All Rights Reserved, <https://jacksum.net>.
 
   This program is free software: you can redistribute it and/or modify it under
@@ -22,7 +22,7 @@
  */
 package net.jacksum.parameters.base;
 
-import net.jacksum.actions.check.ListFilter;
+import net.jacksum.actions.io.verify.ListFilter;
 
 public interface CheckParameters {
 
