@@ -1,8 +1,8 @@
 /*
 
 
-  Jacksum 3.4.0 - a checksum utility in Java
-  Copyright (c) 2001-2022 Dipl.-Inf. (FH) Johann N. Löfflmann,
+  Jacksum 3.7.0 - a checksum utility in Java
+  Copyright (c) 2001-2023 Dipl.-Inf. (FH) Johann N. Löfflmann,
   All Rights Reserved, <https://jacksum.net>.
 
   This program is free software: you can redistribute it and/or modify it under
@@ -22,10 +22,13 @@
  */
 package net.jacksum.parameters.base;
 
+import net.jacksum.parameters.Sequence;
 
 public interface SequenceParameters {
 
     boolean isSequence();
 
-    byte[] getSequenceAsBytes();
+    Sequence getSequence();
+
+    void setSequence(Sequence sequence);
 }
