@@ -23,8 +23,12 @@ package org.bouncycastle.crypto.digests;
         ---------------+--------+-----------+------+------------+
  */
 
+import org.bouncycastle.crypto.CryptoServicePurpose;
+//import org.bouncycastle.crypto.CryptoServicesRegistrar;
 import org.bouncycastle.crypto.ExtendedDigest;
+import org.bouncycastle.crypto.OutputLengthException;
 import org.bouncycastle.util.Arrays;
+import org.bouncycastle.util.Integers;
 import org.bouncycastle.util.Pack;
 
 /**
@@ -40,36 +44,36 @@ import org.bouncycastle.util.Pack;
  * between 1 and 32 bytes.
  */
 public class Blake2sDigest
-    implements ExtendedDigest
+        implements ExtendedDigest
 {
     /**
      * BLAKE2s Initialization Vector
      **/
     private static final int[] blake2s_IV =
-        // Produced from the square root of primes 2, 3, 5, 7, 11, 13, 17, 19.
-        // The same as SHA-256 IV.
-        {
-            0x6a09e667, 0xbb67ae85, 0x3c6ef372,
-            0xa54ff53a, 0x510e527f, 0x9b05688c,
-            0x1f83d9ab, 0x5be0cd19
-        };
+            // Produced from the square root of primes 2, 3, 5, 7, 11, 13, 17, 19.
+            // The same as SHA-256 IV.
+            {
+                    0x6a09e667, 0xbb67ae85, 0x3c6ef372,
+                    0xa54ff53a, 0x510e527f, 0x9b05688c,
+                    0x1f83d9ab, 0x5be0cd19
+            };
 
     /**
      * Message word permutations
      **/
     private static final byte[][] blake2s_sigma =
-        {
-            {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15},
-            {14, 10, 4, 8, 9, 15, 13, 6, 1, 12, 0, 2, 11, 7, 5, 3},
-            {11, 8, 12, 0, 5, 2, 15, 13, 10, 14, 3, 6, 7, 1, 9, 4},
-            {7, 9, 3, 1, 13, 12, 11, 14, 2, 6, 5, 10, 4, 0, 15, 8},
-            {9, 0, 5, 7, 2, 4, 10, 15, 14, 1, 11, 12, 6, 8, 3, 13},
-            {2, 12, 6, 10, 0, 11, 8, 3, 4, 13, 7, 5, 15, 14, 1, 9},
-            {12, 5, 1, 15, 14, 13, 4, 10, 0, 7, 6, 3, 9, 2, 8, 11},
-            {13, 11, 7, 14, 12, 1, 3, 9, 5, 0, 15, 4, 8, 6, 2, 10},
-            {6, 15, 14, 9, 11, 3, 0, 8, 12, 2, 13, 7, 1, 4, 10, 5},
-            {10, 2, 8, 4, 7, 6, 1, 5, 15, 11, 9, 14, 3, 12, 13, 0}
-        };
+            {
+                    {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15},
+                    {14, 10, 4, 8, 9, 15, 13, 6, 1, 12, 0, 2, 11, 7, 5, 3},
+                    {11, 8, 12, 0, 5, 2, 15, 13, 10, 14, 3, 6, 7, 1, 9, 4},
+                    {7, 9, 3, 1, 13, 12, 11, 14, 2, 6, 5, 10, 4, 0, 15, 8},
+                    {9, 0, 5, 7, 2, 4, 10, 15, 14, 1, 11, 12, 6, 8, 3, 13},
+                    {2, 12, 6, 10, 0, 11, 8, 3, 4, 13, 7, 5, 15, 14, 1, 9},
+                    {12, 5, 1, 15, 14, 13, 4, 10, 0, 7, 6, 3, 9, 2, 8, 11},
+                    {13, 11, 7, 14, 12, 1, 3, 9, 5, 0, 15, 4, 8, 6, 2, 10},
+                    {6, 15, 14, 9, 11, 3, 0, 8, 12, 2, 13, 7, 1, 4, 10, 5},
+                    {10, 2, 8, 4, 7, 6, 1, 5, 15, 11, 9, 14, 3, 12, 13, 0}
+            };
 
     private static final int ROUNDS = 10; // to use for Catenas H'
     private static final int BLOCK_LENGTH_BYTES = 64;// bytes
@@ -84,12 +88,14 @@ public class Blake2sDigest
     // Tree hashing parameters:
     // The Tree Hashing Mode is not supported but these are used for
     // the XOF implementation
-	private int fanout = 1; // 0-255
-	private int depth = 1; // 0-255
-	private int leafLength= 0;
-	private long nodeOffset = 0L;
-	private int nodeDepth = 0;
-	private int innerHashLength = 0;
+    private int fanout = 1; // 0-255
+    private int depth = 1; // 0-255
+    private int leafLength = 0;
+    private long nodeOffset = 0L;
+    private int nodeDepth = 0;
+    private int innerHashLength = 0;
+
+    private boolean isLastNode = false;
 
 
     /**
@@ -126,14 +132,28 @@ public class Blake2sDigest
     private int f0 = 0;
 
     // For Tree Hashing Mode, not used here:
-    // private long f1 = 0L; // finalization flag, for last node: ~0L
+    private int f1 = 0;
+    // finalization flag, for last node: ~0L
+
+    // digest purpose
+    private final CryptoServicePurpose purpose;
 
     /**
      * BLAKE2s-256 for hashing.
      */
     public Blake2sDigest()
     {
-        this(256);
+        this(256, CryptoServicePurpose.ANY);
+    }
+
+    /**
+     * Basic sized constructor - size in bits.
+     *
+     * @param digestSize size of digest (in bits)
+     */
+    public Blake2sDigest(int digestSize)
+    {
+        this(digestSize, CryptoServicePurpose.ANY);
     }
 
     public Blake2sDigest(Blake2sDigest digest)
@@ -143,7 +163,7 @@ public class Blake2sDigest
         this.keyLength = digest.keyLength;
         this.key = Arrays.clone(digest.key);
         this.digestLength = digest.digestLength;
-        this.internalState = Arrays.clone(internalState);
+        this.internalState = Arrays.clone(digest.internalState);
         this.chainValue = Arrays.clone(digest.chainValue);
         this.t0 = digest.t0;
         this.t1 = digest.t1;
@@ -156,22 +176,26 @@ public class Blake2sDigest
         this.nodeOffset = digest.nodeOffset;
         this.nodeDepth = digest.nodeDepth;
         this.innerHashLength = digest.innerHashLength;
+        this.purpose = digest.purpose;
     }
 
     /**
      * BLAKE2s for hashing.
      *
      * @param digestBits the desired digest length in bits. Must be a multiple of 8 and less than 256.
+     * @param purpose usage purpose.
      */
-    public Blake2sDigest(int digestBits)
+    public Blake2sDigest(int digestBits, CryptoServicePurpose purpose)
     {
         if (digestBits < 8 || digestBits > 256 || digestBits % 8 != 0)
         {
             throw new IllegalArgumentException(
-                "BLAKE2s digest bit length must be a multiple of 8 and not greater than 256");
+                    "BLAKE2s digest bit length must be a multiple of 8 and not greater than 256");
         }
         digestLength = digestBits / 8;
+        this.purpose = purpose;
 
+//        CryptoServicesRegistrar.checkConstraints(Utils.getDefaultProperties(this, digestBits, purpose));
         init(null, null, null);
     }
 
@@ -186,6 +210,13 @@ public class Blake2sDigest
      */
     public Blake2sDigest(byte[] key)
     {
+        this(key, CryptoServicePurpose.ANY);
+    }
+
+    public Blake2sDigest(byte[] key, CryptoServicePurpose purpose)
+    {
+        this.purpose = purpose;
+//        CryptoServicesRegistrar.checkConstraints(Utils.getDefaultProperties(this, key.length*8, purpose));
         init(null, null, key);
     }
 
@@ -204,27 +235,41 @@ public class Blake2sDigest
      */
     public Blake2sDigest(byte[] key, int digestBytes, byte[] salt,
                          byte[] personalization)
+    {
+        this(key, digestBytes, salt, personalization, CryptoServicePurpose.ANY);
+    }
+    public Blake2sDigest(byte[] key, int digestBytes, byte[] salt,
+                         byte[] personalization, CryptoServicePurpose purpose)
     {
         if (digestBytes < 1 || digestBytes > 32)
         {
             throw new IllegalArgumentException(
-                "Invalid digest length (required: 1 - 32)");
+                    "Invalid digest length (required: 1 - 32)");
         }
         digestLength = digestBytes;
+        this.purpose = purpose;
 
+//        CryptoServicesRegistrar.checkConstraints(Utils.getDefaultProperties(this, digestBytes*8, purpose));
         init(salt, personalization, key);
     }
 
     // XOF root hash parameters
-    Blake2sDigest(int digestBytes, byte[] key, byte[] salt, byte[] personalization, long offset) {
+    Blake2sDigest(int digestBytes, byte[] key, byte[] salt, byte[] personalization, long offset, CryptoServicePurpose purpose)
+    {
         digestLength = digestBytes;
         nodeOffset = offset;
-
+        this.purpose = purpose;
+//        CryptoServicesRegistrar.checkConstraints(Utils.getDefaultProperties(this, digestBytes*8, purpose));
         init(salt, personalization, key);
     }
 
     // XOF internal hash parameters
-    Blake2sDigest(int digestBytes, int hashLength, long offset) {
+    Blake2sDigest(int digestBytes, int hashLength, long offset)
+    {
+        this(digestBytes, hashLength, offset, CryptoServicePurpose.ANY);
+    }
+    Blake2sDigest(int digestBytes, int hashLength, long offset, CryptoServicePurpose purpose)
+    {
         digestLength = digestBytes;
         nodeOffset = offset;
         fanout = 0;
@@ -232,9 +277,29 @@ public class Blake2sDigest
         leafLength = hashLength;
         innerHashLength = hashLength;
         nodeDepth = 0;
+        this.purpose = purpose;
 
+//        CryptoServicesRegistrar.checkConstraints(Utils.getDefaultProperties(this, digestBytes*8, purpose));
         init(null, null, null);
     }
+    Blake2sDigest (byte[] key, byte[] param)
+    {
+        this.purpose = CryptoServicePurpose.ANY;
+        digestLength = param[0];
+        keyLength = param[1];
+        fanout = param[2];
+        depth = param[3];
+        leafLength = Pack.littleEndianToInt(param, 4);
+        nodeOffset |= Pack.littleEndianToInt(param, 8);
+        //xoflength
+        nodeDepth = param[14];
+        innerHashLength = param[15];
+        byte[] salt = new byte[8];
+        byte[] personalization = new byte[8];
+        System.arraycopy(param, 16, salt, 0, 8);
+        System.arraycopy(param, 24, personalization, 0, 8);
+        init(salt, personalization, key);
+    }
 
     // initialize the digest's parameters
     private void init(byte[] salt, byte[] personalization, byte[] key)
@@ -243,16 +308,14 @@ public class Blake2sDigest
 
         if (key != null && key.length > 0)
         {
-            if (key.length > 32)
-            {
-                throw new IllegalArgumentException(
-                        "Keys > 32 bytes are not supported");
-            }
-            this.key = new byte[key.length];
-            System.arraycopy(key, 0, this.key, 0, key.length);
-
             keyLength = key.length;
-            System.arraycopy(key, 0, buffer, 0, key.length);
+            if (keyLength > 32)
+            {
+                throw new IllegalArgumentException("Keys > 32 bytes are not supported");
+            }
+            this.key = new byte[keyLength];
+            System.arraycopy(key, 0, this.key, 0, keyLength);
+            System.arraycopy(key, 0, buffer, 0, keyLength);
             bufferPos = BLOCK_LENGTH_BYTES; // zero padding
         }
 
@@ -260,15 +323,13 @@ public class Blake2sDigest
         {
             chainValue = new int[8];
 
-            chainValue[0] = blake2s_IV[0]
-                ^ (digestLength | (keyLength << 8) | ((fanout << 16) | (depth << 24)));
+            chainValue[0] = blake2s_IV[0] ^ (digestLength | (keyLength << 8) | ((fanout << 16) | (depth << 24)));
             chainValue[1] = blake2s_IV[1] ^ leafLength;
 
-            int nofHi = (int) (nodeOffset >> 32);
-            int nofLo = (int) nodeOffset;
+            int nofHi = (int)(nodeOffset >> 32);
+            int nofLo = (int)nodeOffset;
             chainValue[2] = blake2s_IV[2] ^ nofLo;
-            chainValue[3] = blake2s_IV[3] ^ (nofHi |
-                    (nodeDepth << 16) | (innerHashLength << 24));
+            chainValue[3] = blake2s_IV[3] ^ (nofHi | (nodeDepth << 16) | (innerHashLength << 24));
 
             chainValue[4] = blake2s_IV[4];
             chainValue[5] = blake2s_IV[5];
@@ -276,8 +337,7 @@ public class Blake2sDigest
             {
                 if (salt.length != 8)
                 {
-                    throw new IllegalArgumentException(
-                            "Salt length must be exactly 8 bytes");
+                    throw new IllegalArgumentException("Salt length must be exactly 8 bytes");
                 }
                 this.salt = new byte[8];
                 System.arraycopy(salt, 0, this.salt, 0, salt.length);
@@ -292,12 +352,10 @@ public class Blake2sDigest
             {
                 if (personalization.length != 8)
                 {
-                    throw new IllegalArgumentException(
-                            "Personalization length must be exactly 8 bytes");
+                    throw new IllegalArgumentException("Personalization length must be exactly 8 bytes");
                 }
                 this.personalization = new byte[8];
-                System.arraycopy(personalization, 0, this.personalization, 0,
-                        personalization.length);
+                System.arraycopy(personalization, 0, this.personalization, 0, personalization.length);
 
                 chainValue[6] ^= Pack.littleEndianToInt(personalization, 0);
                 chainValue[7] ^= Pack.littleEndianToInt(personalization, 4);
@@ -313,7 +371,7 @@ public class Blake2sDigest
         internalState[12] = t0 ^ blake2s_IV[4];
         internalState[13] = t1 ^ blake2s_IV[5];
         internalState[14] = f0 ^ blake2s_IV[6];
-        internalState[15] = blake2s_IV[7];// ^ f1 with f1 = 0
+        internalState[15] = f1 ^ blake2s_IV[7];
     }
 
     /**
@@ -370,7 +428,7 @@ public class Blake2sDigest
             if (remainingLength < len)
             { // full buffer + at least 1 byte
                 System.arraycopy(message, offset, buffer, bufferPos,
-                    remainingLength);
+                        remainingLength);
                 t0 += BLOCK_LENGTH_BYTES;
                 if (t0 == 0)
                 { // if message > 2^32
@@ -406,7 +464,7 @@ public class Blake2sDigest
 
         // fill the buffer with left bytes, this might be a full block
         System.arraycopy(message, messagePos, buffer, 0, offset + len
-            - messagePos);
+                - messagePos);
         bufferPos += offset + len - messagePos;
     }
 
@@ -419,7 +477,16 @@ public class Blake2sDigest
      */
     public int doFinal(byte[] out, int outOffset)
     {
+        if (outOffset > (out.length - digestLength))
+        {
+            throw new OutputLengthException("output buffer too short");
+        }
+
         f0 = 0xFFFFFFFF;
+        if(isLastNode)
+        {
+            f1 = 0xFFFFFFFF;
+        }
         t0 += bufferPos;
         // bufferPos may be < 64, so (t0 == 0) does not work
         // for 2^32 < message length > 2^32 - 63
@@ -431,19 +498,13 @@ public class Blake2sDigest
         Arrays.fill(buffer, (byte)0);// Holds eventually the key if input is null
         Arrays.fill(internalState, 0);
 
-        for (int i = 0; i < chainValue.length && (i * 4 < digestLength); i++)
+        int full = digestLength >>> 2, partial = digestLength & 3;
+        Pack.intToLittleEndian(chainValue, 0, full, out, outOffset);
+        if (partial > 0)
         {
-            byte[] bytes = Pack.intToLittleEndian(chainValue[i]);
-
-            if (i * 4 < digestLength - 4)
-            {
-                System.arraycopy(bytes, 0, out, outOffset + i * 4, 4);
-            }
-            else
-            {
-                System.arraycopy(bytes, 0, out, outOffset + i * 4,
-                    digestLength - (i * 4));
-            }
+            byte[] bytes = new byte[4];
+            Pack.intToLittleEndian(chainValue[full], bytes, 0);
+            System.arraycopy(bytes, 0, out, outOffset + digestLength - partial, partial);
         }
 
         Arrays.fill(chainValue, 0);
@@ -461,8 +522,10 @@ public class Blake2sDigest
     {
         bufferPos = 0;
         f0 = 0;
+        f1 = 0;
         t0 = 0;
         t1 = 0;
+        isLastNode = false;
         chainValue = null;
         Arrays.fill(buffer, (byte)0);
         if (key != null)
@@ -479,14 +542,10 @@ public class Blake2sDigest
         initializeInternalState();
 
         int[] m = new int[16];
-        for (int j = 0; j < 16; j++)
-        {
-            m[j] = Pack.littleEndianToInt(message, messagePos + j * 4);
-        }
+        Pack.littleEndianToInt(message, messagePos, m);
 
         for (int round = 0; round < ROUNDS; round++)
         {
-
             // G apply to columns of internalState:m[blake2s_sigma[round][2 *
             // blockPos]] /+1
             G(m[blake2s_sigma[round][0]], m[blake2s_sigma[round][1]], 0, 4, 8, 12);
@@ -510,18 +569,18 @@ public class Blake2sDigest
     private void G(int m1, int m2, int posA, int posB, int posC, int posD)
     {
         internalState[posA] = internalState[posA] + internalState[posB] + m1;
-        internalState[posD] = rotr32(internalState[posD] ^ internalState[posA], 16);
+        internalState[posD] = Integers.rotateRight(internalState[posD] ^ internalState[posA], 16);
         internalState[posC] = internalState[posC] + internalState[posD];
-        internalState[posB] = rotr32(internalState[posB] ^ internalState[posC], 12);
+        internalState[posB] = Integers.rotateRight(internalState[posB] ^ internalState[posC], 12);
         internalState[posA] = internalState[posA] + internalState[posB] + m2;
-        internalState[posD] = rotr32(internalState[posD] ^ internalState[posA], 8);
+        internalState[posD] = Integers.rotateRight(internalState[posD] ^ internalState[posA], 8);
         internalState[posC] = internalState[posC] + internalState[posD];
-        internalState[posB] = rotr32(internalState[posB] ^ internalState[posC], 7);
+        internalState[posB] = Integers.rotateRight(internalState[posB] ^ internalState[posC], 7);
     }
 
-    private int rotr32(int x, int rot)
+    protected void setAsLastNode()
     {
-        return x >>> rot | (x << (32 - rot));
+        isLastNode = true;
     }
 
     /**
@@ -578,4 +637,4 @@ public class Blake2sDigest
             Arrays.fill(salt, (byte)0);
         }
     }
-}
+}
\ No newline at end of file
